@@ -19,7 +19,8 @@
 
 #include "mbexp.h"
 
-void 
+
+void /* Not Used  */
 mb_init(void)
 {
   /* XXXX init gtype system here ? */
@@ -169,6 +170,13 @@ mb_col_unref(MBColor *col)
   col->ref_cnt--;
   if (col->ref_cnt == 0)
     {
+#if defined (USE_XFT) || defined (USE_PANGO)
+      MBPixbuf *pb = col->pb;
+      XftColorFree (pb->dpy,
+                    DefaultVisual(pb->dpy, pb->scr),
+                    DefaultColormap(pb->dpy, pb->scr),
+                    &col->xftcol);
+#endif
       free(col);
       col = NULL;
     }
@@ -319,6 +327,8 @@ _mb_font_load(MBFont *font)
 
   font->metrics = pango_font_get_metrics(font->font, NULL);
 
+  if (font->font != NULL) result = 2;
+
 #elif defined (USE_XFT)
 
   for (i=0; i < MB_N_WEIGHTS; i++)
@@ -337,7 +347,7 @@ _mb_font_load(MBFont *font)
 			    XFT_SLANT, XftTypeInteger , slant,
 			    0);
 
-  if (font->font !=NULL ) result = 2;
+  if (font->font != NULL ) result = 2;
 
 #else
 
@@ -556,6 +566,13 @@ mb_font_set_from_string(MBFont *font, char *spec)
  end:
   free(orig);
 
+  /* Check we can actually load the font ok */
+  if (_mb_font_load(font) == 0)
+    {
+      mb_font_unref(font);
+      return NULL;
+    }
+
   return font;
 }
 
@@ -590,17 +607,27 @@ mb_font_unref(MBFont* font)
 
   if (!font->ref_cnt)
     {
-      if (font->family) free(font->family);
+      if (font->col)
+	mb_col_unref (font->col);
+#ifdef USE_PANGO
+      if (font->fontdes)
+	pango_font_description_free (font->fontdes);
+      if (font->pgo_context)
+	g_object_unref (font->pgo_context);
+      /* The Pango fontmap is owned by Pango apparently */
 
-#if defined (USE_PANGO)
-      /* FIXME To free 
+#elif defined (USE_XFT)
+      ;
+#else
+      if (font->gc) XFreeGC(font->dpy, font->gc);
+#endif
 
-      font->pgo_context
-      font->pgo_fontmap
-      font->fontdes    
+      if (font->family) 
+	free(font->family);
 
-      */
-#endif      
+      _mb_font_free(font);
+
+      free(font);
     }
 }
 
@@ -655,19 +682,53 @@ mb_font_set_point_size(MBFont* font, int points)
 int
 mb_font_set_size_to_pixels(MBFont* font, int max_pixels, int *points_to_try)
 {
-  int i = 0;
-  int pt_sizes[]  = { 72, 48, 32, 24, 20, 18, 16, 14, 12, 11, 10, 9, 8, 7, 6, 5, 0 }; 
+  int    likely_point_size;
+  int    i = 0;
+  double mm_per_pixel, inches_per_pixel;
+
+  /* point stuff not reallu used any more */
+  int    pt_sizes[]  = { 72, 48, 32, 24, 20, 18, 16, 14, 
+			 12, 11, 10, 9, 8, 7, 6, 5, 0 }; 
 
   if (points_to_try == NULL)
     points_to_try = pt_sizes;
 
+  mm_per_pixel = (double)DisplayHeightMM(font->dpy, DefaultScreen(font->dpy))
+                   / DisplayHeight(font->dpy, DefaultScreen(font->dpy));
+
+  /* 1 millimeter = 0.0393700787 inches */
+  
+  inches_per_pixel = (double)mm_per_pixel * 0.03;
+
+  /* 1 inch = 72 PostScript points */
+
+  likely_point_size = max_pixels * inches_per_pixel * 72;
+
+  /*
+    printf("%2f %2f, You asked for %i I reckon thats %ipt\n", 
+    mm_per_pixel, inches_per_pixel, max_pixels, likely_point_size); 
+  */      
+
+  if (font->font) _mb_font_free(font);
+
+  font->pt_size = likely_point_size;
+
+  _mb_font_load(font);
+
+  if (font->font && mb_font_get_height(font) < max_pixels)
+    return 1;
+
   while (pt_sizes[i])
     {
       if (font->font) _mb_font_free(font);
       font->pt_size = pt_sizes[i];
+
       _mb_font_load(font);
+
       if (font->font && mb_font_get_height(font) < max_pixels)
-	return 1;
+	{
+	  return 1;
+	}
       i++;
     }
   
@@ -834,15 +895,19 @@ _clip_some_text (MBFont         *font,
 {
   int len = strlen(txt);
 
+  /* we cant clip single char string */
+  if (len < 2) return 0;
+
   /* XXX RTL case for pango */
 
   if (opts & MB_FONT_RENDER_OPTS_CLIP_TRAIL)
     {
       unsigned char *str = malloc(len+5);
+      memset(str, 0, len+5);
 
-      len += 2;
+      /* len += 2; */
       strcpy(str, txt);
-      
+
       do {
 	  /* go back a glyth */
 	  if (encoding == MB_ENCODING_UTF8)
@@ -856,9 +921,13 @@ _clip_some_text (MBFont         *font,
 
 	  str[len]   = '.'; str[len+1] = '.'; 
 	  str[len+2] = '.'; str[len+3] = '\0'; 
+
+	  /* printf("%s() len %i str is now %s\n", __func__, len, str ); */
       }
       while (mb_font_get_txt_width(font, str, len+3, encoding) > max_width 
-	     && len >= 0);
+	     && len >= 3);
+
+      if (len < 3) len = 0;
 
       free(str);
       return len;
@@ -970,13 +1039,6 @@ _render_some_text (MBFont        *font,
 
 }
 
-/* 
-   renders single line of text
-
-   return num of glypths rendered
-
-   XXX pass flags for clipping ?
-*/
 int
 mb_font_render_simple (MBFont          *font, 
 		       MBDrawable      *drw, 
@@ -1006,6 +1068,8 @@ mb_font_render_simple (MBFont          *font,
   orig_len = len = strlen(text);
 
   str = malloc(len+3);
+  memset(str, 0, len+3);
+
   strcpy(str, text);
 
   render_w = mb_font_get_txt_width(font, str, len, encoding);
@@ -1019,7 +1083,11 @@ mb_font_render_simple (MBFont          *font,
        *
        */
 
+
+
       len = _clip_some_text (font, width, str, encoding, opts);
+
+      if (!len) { free(str); return 0; }
       
       if ((opts & MB_FONT_RENDER_OPTS_CLIP_TRAIL) && len > 3)
 	  want_dots = True;
@@ -1062,6 +1130,55 @@ mb_font_render_simple (MBFont          *font,
   return len;
 }
 
+int
+mb_font_render_simple_get_width (MBFont          *font, 
+				 int              width,
+				 unsigned char   *text,
+				 int              encoding,
+				 MBFontRenderOpts opts )
+{
+  int render_w = 0, len = 0, orig_len = 0;
+  unsigned char *str = NULL;
+  Bool want_dots = False;
+
+  if (text == NULL) return 0;
+
+  if (!_mb_font_is_font_object_fresh (font))
+    _mb_font_load(font);
+
+  orig_len = len = strlen(text);
+
+  str = malloc(len+3);
+  memset(str, 0, len+3);
+
+  strcpy(str, text);
+
+  render_w = mb_font_get_txt_width(font, str, len, encoding);
+
+  if (render_w > width)
+    {
+      len = _clip_some_text (font, width, str, encoding, opts);
+
+      if (!len) { free(str); return 0; }
+      
+      if ((opts & MB_FONT_RENDER_OPTS_CLIP_TRAIL) && len > 3)
+	  want_dots = True;
+    }
+  
+  if ((opts & MB_FONT_RENDER_OPTS_CLIP_TRAIL) && want_dots)
+    {
+      str[len] = '.'; str[len+1] = '.'; str[len+2] = '.'; str[len+3] = '\0'; 
+
+      len += 3; 
+    }
+
+  render_w = mb_font_get_txt_width(font, str, len, encoding);
+  
+  free(str);
+  
+  return render_w;
+}
+
 
 /* 
    Layout defualt to drawable size. 
@@ -1356,7 +1473,7 @@ mb_drawable_new(MBPixbuf *pb, int width, int height)
    /* todo check for error if pixmap cant be created */
 #if defined (USE_XFT) || defined (USE_PANGO)
   drw->xftdraw = XftDrawCreate(pb->dpy, (Drawable) drw->xpixmap, 
-			       DefaultVisual(pb->dpy, pb->scr),
+			       pb->vis,
 			       DefaultColormap(pb->dpy, pb->scr));
 #endif
    
